﻿Public Class CalcFileWriter

#Region " Declarations "
  Private _model As CalcModel
  Private _type As VariableType
  Private _data() As Byte
  Private _checksum As Integer

  Public Enum CalcModel
    TI8X = 1
    TI89 = 2
    TI89Ti = 3
    TI92P = 4
    TIVoyage200 = 5
  End Enum

  Public Enum VariableType
    AppVar = 1
    Certificate = 2
    FlashApplication = 3
    OSUpgrade = 4
  End Enum
#End Region

#Region " Public Properties "
  Public ReadOnly Property Model() As CalcModel
    Get
      Return _model
    End Get
  End Property

  Public ReadOnly Property Type() As VariableType
    Get
      Return _type
    End Get
  End Property

  Public Property Data() As Byte()
    Get
      Return _data
    End Get
    Set(ByVal value As Byte())
      _data = value
    End Set
  End Property
#End Region

#Region " Constructors/Teardown "
  Public Sub New(ByVal model As CalcModel, ByVal type As VariableType)
    _model = model
    _type = type
  End Sub
#End Region

#Region " Public Methods "
  Public Sub Save(ByVal fileName As String)
    Dim f As New IO.FileStream(fileName, IO.FileMode.Create)

    'Depending on variable type, write the header
    Select Case _type
      Case VariableType.Certificate
        Dim headerData1() As Byte = {&H2A, &H2A, &H54, &H49, &H46, &H4C, &H2A, &H2A}
        Dim headerData2(39) As Byte

        f.Write(headerData1, 0, headerData1.Length)
        f.Write(headerData2, 0, headerData2.Length)
        f.WriteByte(&H73)
        f.WriteByte(&H25)
        f.Write(headerData2, 0, 24)

        'Write the size and data
        Dim size As Integer
        If _data IsNot Nothing Then size = _data.Length
        f.WriteByte(Convert.ToByte(size And &HFF))
        f.WriteByte(Convert.ToByte((size And &HFF00) >> 8))
        f.WriteByte(Convert.ToByte((size And &HFF0000) >> 16))
        f.WriteByte(Convert.ToByte((size And &HFF000000) >> 24))
        f.Write(_data, 0, _data.Length)
      Case VariableType.AppVar
        Dim headerData1() As Byte = {&H2A, &H2A, &H54, &H49, &H38, &H33, &H46, &H2A, &H1A, &HA}
        Dim headerData2(42) As Byte

        f.Write(headerData1, 0, headerData1.Length)
        f.Write(headerData2, 0, headerData2.Length)

        'Write the data length for the first header
        Dim headerSize As Integer = 17
        If _data IsNot Nothing Then headerSize += _data.Length
        f.WriteByte(Convert.ToByte(headerSize And &HFF))
        f.WriteByte(Convert.ToByte((headerSize And &HFF00) >> 8))

        'Write the second header (all this is checksummed)
        _checksum = 0
        f.WriteByte(_AddToChecksum(&HD))
        f.WriteByte(_AddToChecksum(0))
        Dim size As Integer
        If _data IsNot Nothing Then size = _data.Length
        f.WriteByte(_AddToChecksum(Convert.ToByte(size And &HFF)))
        f.WriteByte(_AddToChecksum(Convert.ToByte((size And &HFF00) >> 8)))
        f.WriteByte(_AddToChecksum(&H15))
        Dim variableName As String = IO.Path.GetFileNameWithoutExtension(fileName).ToUpper()
        For i As Integer = 0 To Math.Min(variableName.Length, 8) - 1
          f.WriteByte(_AddToChecksum(Convert.ToByte(variableName(i))))
        Next
        For i As Integer = 0 To (8 - variableName.Length) - 1
          f.WriteByte(0)
        Next
        f.WriteByte(0)
        f.WriteByte(0)
        f.WriteByte(_AddToChecksum(Convert.ToByte(size And &HFF)))
        f.WriteByte(_AddToChecksum(Convert.ToByte((size And &HFF00) >> 8)))
        f.Write(_data, 0, _data.Length)

        For i As Integer = 0 To _data.Length - 1
          _AddToChecksum(_data(i))
        Next

        f.WriteByte(Convert.ToByte(_checksum And &HFF))
        f.WriteByte(Convert.ToByte((_checksum And &HFF00) >> 8))
    End Select

    f.Close()
  End Sub
#End Region

#Region " Local Methods "
  Private Function _AddToChecksum(ByVal b As Byte) As Byte
    _checksum = Convert.ToInt32((_checksum + b) And &HFFFF)

    Return b
  End Function
#End Region

End Class
